import pytest


@pytest.fixture
def procfs_timeline():
    """
    Timeline of procfs report for the tests
    """

    return [
        {
            "timestamp": "2021-09-14T12:37:37.168817",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:37.669237",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:38.170142",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:38.670338",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:39.171321",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:39.671572",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:40.172503",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:40.672693",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:41.173552",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:41.673815",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:42.174560",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:42.674690",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:43.175441",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:43.675743",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:44.176551",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:44.677307",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:45.178049",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:45.678310",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:46.179120",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:46.679308",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:47.180223",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:47.680468",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:48.181316",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:48.681683",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:49.182522",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:49.682731",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:50.183680",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:50.683812",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:51.184792",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:51.685027",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:52.185709",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:52.686065",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.610000000000014,
        },
        {
            "timestamp": "2021-09-14T12:37:53.186929",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.36,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:53.687190",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:54.188031",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.600000000000012,
        },
        {
            "timestamp": "2021-09-14T12:37:54.688674",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.59000000000001,
        },
        {
            "timestamp": "2021-09-14T12:37:55.189489",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.59000000000001,
        },
        {
            "timestamp": "2021-09-14T12:37:55.690299",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.59000000000001,
        },
        {
            "timestamp": "2021-09-14T12:37:56.191124",
            "sensor": "formula_group",
            "target": ["firefox_cgroup", "emacs_cgroup",
                       "zsh_cgroup", "mongo_cgroup"],
            "usage": {
                "firefox_cgroup": 8.35,
                "emacs_cgroup": 5.52,
                "zsh_cgroup": 0.01,
                "mongo_cgroup": 0.64,
            },
            "global_cpu_usage": 27.59000000000001,
        },
    ]


@pytest.fixture
def power_timeline():
    """
    Timeline of procfs report for the tests
    """

    return [
        {
            "timestamp": "2021-09-14T12:37:37.168817",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:37.669237",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:38.170142",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:38.670338",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:39.171321",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:39.671572",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:40.172503",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:40.672693",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:41.173552",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:41.673815",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:42.174560",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:42.674690",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:43.175441",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:43.675743",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:44.176551",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:44.677307",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:45.178049",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:45.678310",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:46.179120",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:46.679308",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:47.180223",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:47.680468",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:48.181316",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:48.681683",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:49.182522",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:49.682731",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:50.183680",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:50.683812",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:51.184792",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:51.685027",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:52.185709",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:52.686065",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:53.186929",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:53.687190",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:54.188031",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:54.688674",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:55.189489",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:55.690299",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },
        {
            "timestamp": "2021-09-14T12:37:56.191124",
            "sensor": "formula_group",
            "target": "all",
            "power": 42,
        },

    ]
